@@ -110,7 +110,7 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	list<string> datadirs = PortableKDE::globalDirs("data");
 	list<string>::iterator it;
 
-	for(it = datadirs.begin(); it != datadirs.end(); it++)
+	for(it = datadirs.begin(); it != datadirs.end(); ++it)
 	{
 		string common = *it;
 		common += "artsbuilder";
@@ -135,14 +135,14 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 
 	}
 #endif
-	
+
 	arts_debug("PORT: structure");
 	structure = new Structure();
 	arts_debug("PORT: structure ok");
 	//ModuleList = structure->getModuleList();
 
 	mainDock = createDockWidget("mainDockWidget", 0, 0, "main_dock_widget");
-	
+
 	arts_debug("PORT: modulewidget");
 	modulewidget = new ModuleWidget(structure, mainDock, "mwidget");
 	mainDock->setWidget(modulewidget);
@@ -153,10 +153,10 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	mainDock->setDockSite(KDockWidget::DockCorner);
 	// forbit docking abilities of module widget itself
 	mainDock->setEnableDocking(KDockWidget::DockNone);
-	
+
 	setView(mainDock);
 	setMainDockWidget(mainDock);
-	
+
 	propertyDock = createDockWidget("propertyDock", 0, 0, i18n("Port Properties"));
 
 	propertyPanel = new PropertyPanel(propertyDock, "ppanel");
@@ -173,11 +173,11 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 			modulewidget,  SLOT  (selectPort(ModulePort *)));
 	connect(modulewidget,  SIGNAL(componentSelected(StructureComponent *)),
 			propertyPanel, SLOT  (setSelectedComponent(StructureComponent *)));
-	
+
 	// connection
 	connect(propertyPanel, SIGNAL(startConnection(ModulePort *)),
 			modulewidget,  SLOT  (startConnection(ModulePort *)));
-	
+
 	// port properties changed
 	connect(propertyPanel, SIGNAL(portPropertiesChanged(ModulePort *)),
 			modulewidget,  SLOT  (portPropertiesChanged(ModulePort *)));
@@ -185,9 +185,9 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	arts_debug("PORT: setcanvas");
 	structure->setCanvas(modulewidget);
 	arts_debug("PORT: setcanvas ok");
-	
+
 	mbroker_updateCount = 0;
-		
+
 	arts_debug("PORT: menumaker");
 	menumaker = new MenuMaker(new KActionMenu(i18n("Modules"), actionCollection(), "modulesmenu"));
 	//menumaker->addCategory("&Gui", "Gui_");
@@ -281,7 +281,7 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 		connect(viewmenu, SIGNAL(aboutToShow()), this, SLOT(viewMenuAboutToShow()));
 	else
 		arts_debug("view menu not found!");
-	
+
 	m_filename = QString::null;
 	setModified(false);
 
@@ -292,7 +292,7 @@ void ArtsBuilderWindow::setupActions()
 {
 	// File menu
 	KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
-	
+
 	(void)new KAction(i18n("Open Session..."), 0, this, SLOT(openSession()),
 					  actionCollection(), "file_open_session");
 	KStdAction::open(this, SLOT(open()), actionCollection());
@@ -306,7 +306,7 @@ void ArtsBuilderWindow::setupActions()
 					  actionCollection(), "file_execute_structure");
 	(void)new KAction(i18n("&Rename Structure..."), Qt::CTRL + Qt::Key_R, this, SLOT(rename()),
 					  actionCollection(), "file_rename_structure");
-	(void)new KAction(i18n("&Publish Structure!"), Qt::CTRL + Qt::Key_P, this, SLOT(publish()),
+	(void)new KAction(i18n("&Publish Structure"), Qt::CTRL + Qt::Key_P, this, SLOT(publish()),
 					  actionCollection(), "file_publish_structure");
 	KStdAction::quit(this, SLOT(close()), actionCollection());
 
@@ -346,7 +346,7 @@ void ArtsBuilderWindow::setupActions()
 void ArtsBuilderWindow::fillModuleMenu()
 {
 	long updateCount = 3;  /* PORT: automatic update of menues missing */
-	
+
 	if(updateCount != mbroker_updateCount)
 	{
 		mbroker_updateCount = updateCount;
@@ -360,12 +360,12 @@ void ArtsBuilderWindow::fillModuleMenu()
 
 		vector<Arts::TraderOffer>::iterator i;
 		long n = 1;		/* TODO:PORT: is this necessary? I think not */
-		for(i = offers->begin(); i != offers->end(); i++)
+		for(i = offers->begin(); i != offers->end(); ++i)
 		{
 			Arts::TraderOffer& offer = *i;
 			string name = offer.interfaceName();
 			menumaker->addItem(name.c_str(),n++);
-			
+
 			/* PORT: templates missing
 			if(strncmp(name, "template_", strlen("template_")) == 0)
 			{
@@ -401,7 +401,7 @@ void ArtsBuilderWindow::fillModuleMenu()
 			{
 				const char *name = (*Modules)[i];
 				menumaker->addItem(name, i);
-			
+
 				if(strncmp(name, "template_", strlen("template_")) == 0)
 				{
 					char *xname = strdup(&name[strlen("template_")]);
@@ -497,8 +497,8 @@ void ArtsBuilderWindow::open(QString filename)
 		if(!structure->valid())
 		{
 			KMessageBox::sorry(this,
-			i18n("The structure couldn't be loaded correctly. Maybe some of\n"
-				 "the modules used in the file aren't available in this\n"
+			i18n("The structure could not be loaded correctly. Maybe some of\n"
+				 "the modules used in the file are not available in this\n"
 				 "version of aRts."),
 				i18n("Arts Warning"));
 		}
@@ -549,7 +549,7 @@ void ArtsBuilderWindow::openExample()
 		KMessageBox::sorry(
 			this,
 			i18n("Unable to find the examples folder.\nUsing the current folder instead."),
-			i18n("Arts Warning"));
+			i18n("aRts Warning"));
 
 	open(getOpenFilename("*.arts", QFile::encodeName(dir)));
 }
@@ -587,17 +587,17 @@ bool ArtsBuilderWindow::save(QString filename)
 		KMessageBox::sorry(this,
 			i18n("The file '%1' could not be opened for writing: %2")
 			.arg(filename).arg(strerror(file.status())),
-			i18n("Arts Warning"));
+			i18n("aRts Warning"));
 		return false;
 	}
 
 	structure->saveInto(file.fstream());
-	
+
 	if(!file.close()) {
 		KMessageBox::sorry(this,
 			i18n("Saving to file '%1' could not be finished correctly: %2")
 			.arg(filename).arg(strerror(file.status())),
-			i18n("Arts Warning"));
+			i18n("aRts Warning"));
 		return false;
 	}
 
@@ -635,7 +635,7 @@ void ArtsBuilderWindow::rename()
 		arts_debug("rename OK...");
 		structure->rename(name.local8Bit());
 	}
-	
+
 	setModified(true);
 }
 
@@ -683,8 +683,8 @@ void ArtsBuilderWindow::execute()
 		execDlg = 0;
 
 		KMessageBox::sorry(this,
-			i18n("Couldn't execute your structure. Make sure that the\n"
-				 "sound server (artsd) is running.\n"), i18n("Arts Warning"));
+			i18n("Could not execute your structure. Make sure that the\n"
+				 "sound server (artsd) is running.\n"), i18n("aRts Warning"));
 	}
 }
 
@@ -872,7 +872,7 @@ ArtsBuilderApp::ArtsBuilderApp(QString filename)
 	} else {
 		KMessageBox::sorry(0,
 		   i18n("The specified file '%1' does not exist.").arg(filename),
-		   i18n("Arts Warning"));
+		   i18n("aRts Warning"));
 	}
 }
 
@@ -904,7 +904,7 @@ void ArtsBuilderApp::end()
 
 static KCmdLineOptions options[] =
 {
-	{ "+[file]", I18N_NOOP("Optional .arts file to be loaded."), 0 },
+	{ "+[file]", I18N_NOOP("Optional .arts file to be loaded"), 0 },
 	KCmdLineLastOption
 };
 
@@ -917,7 +917,7 @@ int main(int argc, char **argv)
 	KAboutData aboutData("artsbuilder",
 						 I18N_NOOP("artsbuilder"),
 						 ARTS_VERSION,
-						 I18N_NOOP("Arts synthesizer designer"),
+						 I18N_NOOP("aRts synthesizer designer"),
 						 KAboutData::License_GPL,
 						 "(C) 1998-2001, Stefan Westerfeld",
 						 I18N_NOOP("The analog real-time synthesizer graphical design tool."),
