@@ -15,17 +15,26 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     */
 
+#ifndef ARTS_SEQUENCEUTILS_H
+#define ARTS_SEQUENCEUTILS_H
+
 #include <vector>
 #include <string>
 
 int parse_line(const char *in, char *& cmd, char *& param);
 
-void sqprintf(std::vector<std::string> *list, const char *fmt, ...);
+void sqprintf(std::vector<std::string> *list, const char *fmt, ...)
+#ifdef __GNUC__
+    __attribute__ (( format (printf, 2, 3)))
+#endif
+;
 void addSubStringSeq(std::vector<std::string> *target, const std::vector<std::string> *source);
 void appendStringSeq(std::vector<std::string> *target, const std::vector<std::string> *source);
 int parse_line(const std::string& in, std::string& cmd, std::string& param);
 std::vector<std::string> *getSubStringSeq(const std::vector<std::string> *seq,unsigned long& i);
+
+#endif /* ARTS_SEQUENCEUTILS_H */
