@@ -1,8 +1,6 @@
 /***************************************************************************
-                          tageditor.cpp  -  description
-                             -------------------
     begin                : Sat Sep 7 2002
-    copyright            : (C) 2002, 2003 by Scott Wheeler
+    copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
  ***************************************************************************/
 
@@ -15,6 +13,12 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "tageditor.h"
+#include "collectionlist.h"
+#include "playlistitem.h"
+#include "tag.h"
+#include "actioncollection.h"
+
 #include <kcombobox.h>
 #include <klineedit.h>
 #include <knuminput.h>
@@ -23,41 +27,144 @@
 #include <kconfig.h>
 #include <klocale.h>
 #include <kdebug.h>
+#include <kiconloader.h>
+#include <kactionclasses.h>
 
 #include <qlabel.h>
 #include <qcheckbox.h>
 #include <qlayout.h>
 #include <qdir.h>
+#include <qvalidator.h>
+#include <qtooltip.h>
+#include <qeventloop.h>
 
 #include <id3v1genres.h>
 
-#include "tageditor.h"
-#include "collectionlist.h"
-#include "playlistitem.h"
+using namespace ActionCollection;
+
+class FileNameValidator : public QValidator
+{
+public:
+    FileNameValidator(QObject *parent, const char *name = 0) :
+	QValidator(parent, name) {}
+
+    virtual void fixup(QString &s) const
+    {
+	s.remove('/');
+    }
+
+    virtual State validate(QString &s, int &) const
+    {
+	if(s.find('/') != -1)
+	   return Invalid;
+	return Acceptable;
+    }
+};
+
+class FileBoxToolTip : public QToolTip
+{
+public:
+    FileBoxToolTip(TagEditor *editor, QWidget *widget) :
+	QToolTip(widget), m_editor(editor) {}
+protected:
+    virtual void maybeTip(const QPoint &)
+    {
+	tip(parentWidget()->rect(), m_editor->items().first()->file().absFilePath());
+    }
+private:
+    TagEditor *m_editor;
+};
+
+class FixedHLayout : public QHBoxLayout
+{
+public:
+    FixedHLayout(QWidget *parent, int margin = 0, int spacing = -1, const char *name = 0) :
+	QHBoxLayout(parent, margin, spacing, name),
+	m_width(-1) {}
+    FixedHLayout(QLayout *parentLayout, int spacing = -1, const char *name = 0) :
+	QHBoxLayout(parentLayout, spacing, name),
+	m_width(-1) {}
+    void setWidth(int w = -1)
+    {
+	m_width = w == -1 ? QHBoxLayout::minimumSize().width() : w;
+    }
+    virtual QSize minimumSize() const
+    {
+	QSize s = QHBoxLayout::minimumSize();
+	s.setWidth(m_width);
+	return s;
+    }
+private:
+    int m_width;
+};
+
+class CollectionObserver : public PlaylistObserver
+{
+public:
+    CollectionObserver(TagEditor *parent) :
+	PlaylistObserver(CollectionList::instance()),
+	m_parent(parent)
+    {
+    }
+
+    virtual void updateData()
+    {
+	if(m_parent && m_parent->m_currentPlaylist)
+	    m_parent->slotSetItems(m_parent->m_currentPlaylist->selectedItems());
+    }
+
+    virtual void updateCurrent() {}
+
+private:
+    TagEditor *m_parent;
+};
 
 ////////////////////////////////////////////////////////////////////////////////
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-TagEditor::TagEditor(QWidget *parent, const char *name) : QWidget(parent, name),
-							  m_currentPlaylist(0)
+TagEditor::TagEditor(QWidget *parent, const char *name) :
+    QWidget(parent, name),
+    m_currentPlaylist(0),
+    m_observer(0),
+    m_performingSave(false)
 {
+    setupActions();
     setupLayout();
     readConfig();
     m_dataChanged = false;
+    m_collectionChanged = false;
 }
 
 TagEditor::~TagEditor()
 {
+    delete m_observer;
     saveConfig();
 }
 
+void TagEditor::setupObservers()
+{
+    m_observer = new CollectionObserver(this);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // public slots
 ////////////////////////////////////////////////////////////////////////////////
 
 void TagEditor::slotSetItems(const PlaylistItemList &list)
 {
+    if(m_performingSave)
+	return;
+
+    // Store the playlist that we're setting because saveChangesPrompt
+    // can delete the PlaylistItems in list.
+
+    Playlist *itemPlaylist = 0;
+    if(!list.isEmpty())
+	itemPlaylist = list.first()->playlist();
+
+    bool hadPlaylist = m_currentPlaylist != 0;
+
     saveChangesPrompt();
 
     if(m_currentPlaylist) {
@@ -65,17 +172,28 @@ void TagEditor::slotSetItems(const PlaylistItemList &list)
 		   this, SLOT(slotItemRemoved(PlaylistItem *)));
     }
 
-    m_currentPlaylist = list.isEmpty() ? 0 : static_cast<Playlist *>(list.first()->listView());
+    if(hadPlaylist && !m_currentPlaylist || !itemPlaylist) {
+	m_currentPlaylist = 0;
+	m_items.clear();
+    }
+    else {
+	m_currentPlaylist = itemPlaylist;
+
+	// We can't use list here, it may not be valid
+
+	m_items = itemPlaylist->selectedItems();
+    }
 
     if(m_currentPlaylist) {
 	connect(m_currentPlaylist, SIGNAL(signalAboutToRemove(PlaylistItem *)),
 		this, SLOT(slotItemRemoved(PlaylistItem *)));
+	connect(m_currentPlaylist, SIGNAL(destroyed()), this, SLOT(slotPlaylistRemoved()));
     }
 
-    m_items = list;
-
     if(isVisible())
 	slotRefresh();
+    else
+	m_collectionChanged = true;
 }
 
 void TagEditor::slotRefresh()
@@ -85,7 +203,7 @@ void TagEditor::slotRefresh()
     // the most common case -- is to just process the first item.  Then we
     // check after that to see if there are other m_items and adjust accordingly.
 
-    if(m_items.isEmpty() || !m_items.first()->tag()) {
+    if(m_items.isEmpty() || !m_items.first()->file().tag()) {
 	slotClear();
 	setEnabled(false);
 	return;
@@ -95,13 +213,18 @@ void TagEditor::slotRefresh()
 
     PlaylistItem *item = m_items.first();
 
-    Tag *tag = item->tag();
+    Tag *tag = item->file().tag();
 	
+    QFileInfo fi(item->file().absFilePath());
+    if(!fi.isWritable() && m_items.count() == 1)
+	setEnabled(false);
+
     m_artistNameBox->setEditText(tag->artist());
     m_trackNameBox->setText(tag->title());
     m_albumNameBox->setEditText(tag->album());
 
-    m_fileNameBox->setText(item->fileName());
+    m_fileNameBox->setText(item->file().fileInfo().fileName());
+    new FileBoxToolTip(this, m_fileNameBox);
     m_bitrateBox->setText(QString::number(tag->bitrate()));
     m_lengthBox->setText(tag->lengthString());
 
@@ -153,7 +276,7 @@ void TagEditor::slotRefresh()
 	}
 	else {
 	    for(; it != m_items.end(); ++it) {
-		tag = (*it)->tag();
+		tag = (*it)->file().tag();
 
 		if(tag) {
 
@@ -235,6 +358,16 @@ void TagEditor::slotClear()
 
 void TagEditor::slotUpdateCollection()
 {
+    if(isVisible())
+	updateCollection();
+    else
+	m_collectionChanged = true;
+}
+
+void TagEditor::updateCollection()
+{
+    m_collectionChanged = false;
+        
     CollectionList *list = CollectionList::instance();
 
     if(!list)
@@ -257,16 +390,19 @@ void TagEditor::slotUpdateCollection()
 
 void TagEditor::readConfig()
 {
-    KConfig *config = KGlobal::config();
-    { // combo box completion modes
-	KConfigGroupSaver saver(config, "TagEditor");
-	if(m_artistNameBox && m_albumNameBox) {
-	    readCompletionMode(config, m_artistNameBox, "ArtistNameBoxMode");
-	    readCompletionMode(config, m_albumNameBox, "AlbumNameBoxMode");
-	    readCompletionMode(config, m_genreBox, "GenreBoxMode");
-        }
+    // combo box completion modes
+	
+    KConfigGroup config(KGlobal::config(), "TagEditor");
+    if(m_artistNameBox && m_albumNameBox) {
+	readCompletionMode(&config, m_artistNameBox, "ArtistNameBoxMode");
+	readCompletionMode(&config, m_albumNameBox, "AlbumNameBoxMode");
+	readCompletionMode(&config, m_genreBox, "GenreBoxMode");
     }
 
+    bool show = config.readBoolEntry("Show", false);
+    action<KToggleAction>("showEditor")->setChecked(show);
+    setShown(show);
+
     TagLib::StringList genres = TagLib::ID3v1::genreList();
 
     for(TagLib::StringList::ConstIterator it = genres.begin(); it != genres.end(); ++it)
@@ -279,7 +415,7 @@ void TagEditor::readConfig()
     m_genreBox->completionObject()->setItems(m_genreList);
 }
 
-void TagEditor::readCompletionMode(KConfig *config, KComboBox *box, const QString &key)
+void TagEditor::readCompletionMode(KConfigBase *config, KComboBox *box, const QString &key)
 {
     KGlobalSettings::Completion mode =
 	KGlobalSettings::Completion(config->readNumEntry(key, KGlobalSettings::CompletionAuto));
@@ -289,24 +425,33 @@ void TagEditor::readCompletionMode(KConfig *config, KComboBox *box, const QStrin
 
 void TagEditor::saveConfig()
 {
-    KConfig *config = KGlobal::config();
-    { // combo box completion modes
-        KConfigGroupSaver saver(config, "TagEditor");
-        if(m_artistNameBox && m_albumNameBox) {
-	    config->writeEntry("ArtistNameBoxMode", m_artistNameBox->completionMode());
-	    config->writeEntry("AlbumNameBoxMode", m_albumNameBox->completionMode());
-	    config->writeEntry("GenreBoxMode", m_genreBox->completionMode());
-        }
-    }
+    // combo box completion modes
 
+    KConfigGroup config(KGlobal::config(), "TagEditor");
+
+    if(m_artistNameBox && m_albumNameBox) {
+	config.writeEntry("ArtistNameBoxMode", m_artistNameBox->completionMode());
+	config.writeEntry("AlbumNameBoxMode", m_albumNameBox->completionMode());
+	config.writeEntry("GenreBoxMode", m_genreBox->completionMode());
+    }
+    config.writeEntry("Show", action<KToggleAction>("showEditor")->isChecked());
+}
+
+void TagEditor::setupActions()
+{
+    KToggleAction *show = new KToggleAction(i18n("Show &Tag Editor"), "edit", 0, actions(), "showEditor");
+    show->setCheckedState(i18n("Hide &Tag Editor"));
+    connect(show, SIGNAL(toggled(bool)), this, SLOT(setShown(bool)));
+
+    new KAction(i18n("&Save"), "filesave", "CTRL+t", this, SLOT(slotSave()), actions(), "saveItem");
 }
 
 void TagEditor::setupLayout()
 {
-    static const int horizontalSpacing = 10;
+    static const int horizontalSpacing = 12;
     static const int verticalSpacing = 2;
 
-    QHBoxLayout *layout = new QHBoxLayout(this, 2, horizontalSpacing);
+    QHBoxLayout *layout = new QHBoxLayout(this, 6, horizontalSpacing);
 
     //////////////////////////////////////////////////////////////////////////////
     // define two columns of the bottem layout
@@ -321,19 +466,20 @@ void TagEditor::setupLayout()
     // put stuff in the left column -- all of the field names are class wide
     //////////////////////////////////////////////////////////////////////////////
     { // just for organization
-        m_artistNameBox = new KComboBox(true, this, "artistNameBox");
+    	
+	m_artistNameBox = new KComboBox(true, this, "artistNameBox");
 	m_artistNameBox->setCompletionMode(KGlobalSettings::CompletionAuto);
-	addItem(i18n("&Artist name:"), m_artistNameBox, leftColumnLayout);
+	addItem(i18n("&Artist name:"), m_artistNameBox, leftColumnLayout, "personal");
 
         m_trackNameBox = new KLineEdit(this, "trackNameBox");
-	addItem(i18n("&Track name:"), m_trackNameBox, leftColumnLayout);
+	addItem(i18n("&Track name:"), m_trackNameBox, leftColumnLayout, "player_play");
 
-        m_albumNameBox = new KComboBox(true, this, "albumNameBox");
+	m_albumNameBox = new KComboBox(true, this, "albumNameBox");
 	m_albumNameBox->setCompletionMode(KGlobalSettings::CompletionAuto);
-	addItem(i18n("Album &name:"), m_albumNameBox, leftColumnLayout);
+	addItem(i18n("Album &name:"), m_albumNameBox, leftColumnLayout, "cdrom_unmount");
 
         m_genreBox = new KComboBox(true, this, "genreBox");
-	addItem(i18n("&Genre:"), m_genreBox, leftColumnLayout);
+	addItem(i18n("&Genre:"), m_genreBox, leftColumnLayout, "knotify");
 
         // this fills the space at the bottem of the left column
         leftColumnLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum,
@@ -343,50 +489,69 @@ void TagEditor::setupLayout()
     // put stuff in the right column
     //////////////////////////////////////////////////////////////////////////////
     { // just for organization
+	
+	QHBoxLayout *fileNameLayout = new QHBoxLayout(rightColumnLayout,
+						      horizontalSpacing);
 
-        m_fileNameBox = new KLineEdit(this, "fileNameBox");
+	m_fileNameBox = new KLineEdit(this, "fileNameBox");
+	m_fileNameBox->setValidator(new FileNameValidator(m_fileNameBox));	
+
+	QLabel *fileNameIcon = new QLabel(this);
+	fileNameIcon->setPixmap(SmallIcon("sound"));
 	QWidget *fileNameLabel = addHidden(new QLabel(m_fileNameBox, i18n("&File name:"), this));
 
-        rightColumnLayout->addWidget(fileNameLabel);
-        rightColumnLayout->addWidget(addHidden(m_fileNameBox));
+	fileNameLayout->addWidget(addHidden(fileNameIcon));
+	fileNameLayout->addWidget(fileNameLabel);
+	fileNameLayout->setStretchFactor(fileNameIcon, 0);
+	fileNameLayout->setStretchFactor(fileNameLabel, 0);
+	fileNameLayout->insertStretch(-1, 1);
+	rightColumnLayout->addWidget(addHidden(m_fileNameBox));
 
         { // lay out the track row
-            QHBoxLayout *trackRowLayout = new QHBoxLayout(rightColumnLayout,
-							  horizontalSpacing);
+            FixedHLayout *trackRowLayout = new FixedHLayout(rightColumnLayout,
+							    horizontalSpacing);
 
-            m_trackSpin = new KIntSpinBox(0, 255, 1, 0, 10, this, "trackSpin");
+	    m_trackSpin = new KIntSpinBox(0, 255, 1, 0, 10, this, "trackSpin");
 	    addItem(i18n("T&rack:"), m_trackSpin, trackRowLayout);
 	    m_trackSpin->installEventFilter(this);
 
-            trackRowLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding,
+	    trackRowLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding,
 						    QSizePolicy::Minimum));
 
-            m_yearSpin = new KIntSpinBox(0, 9999, 1, 0, 10, this, "yearSpin");
+	    m_yearSpin = new KIntSpinBox(0, 9999, 1, 0, 10, this, "yearSpin");
 	    addItem(i18n("&Year:"), m_yearSpin, trackRowLayout);
 	    m_yearSpin->installEventFilter(this);
 
-            trackRowLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding,
+	    trackRowLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding,
 						    QSizePolicy::Minimum));
 
-            trackRowLayout->addWidget(addHidden(new QLabel(i18n("Length:"), this)));
-            m_lengthBox = new KLineEdit(this, "lengthBox");
-            m_lengthBox->setMaximumWidth(50);
-            m_lengthBox->setReadOnly(true);
-            trackRowLayout->addWidget(addHidden(m_lengthBox));
+	    trackRowLayout->addWidget(addHidden(new QLabel(i18n("Length:"), this)));
+	    m_lengthBox = new KLineEdit(this, "lengthBox");
+	    // addItem(i18n("Length:"), m_lengthBox, trackRowLayout);
+	    m_lengthBox->setMaximumWidth(50);
+	    m_lengthBox->setAlignment(Qt::AlignRight);
+	    m_lengthBox->setReadOnly(true);
+	    trackRowLayout->addWidget(addHidden(m_lengthBox));
 
-            trackRowLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding,
+	    trackRowLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding,
 						    QSizePolicy::Minimum));
 
-            trackRowLayout->addWidget(addHidden(new QLabel(i18n("Bitrate:"), this)));
-            m_bitrateBox = new KLineEdit(this, "bitrateBox");
-            m_bitrateBox->setMaximumWidth(50);
-            m_bitrateBox->setReadOnly(true);
-            trackRowLayout->addWidget(addHidden(m_bitrateBox));
+	    trackRowLayout->addWidget(addHidden(new QLabel(i18n("Bitrate:"), this)));
+	    m_bitrateBox = new KLineEdit(this, "bitrateBox");
+	    // addItem(i18n("Bitrate:"), m_bitrateBox, trackRowLayout);
+	    m_bitrateBox->setMaximumWidth(50);
+	    m_bitrateBox->setAlignment(Qt::AlignRight);
+	    m_bitrateBox->setReadOnly(true);
+	    trackRowLayout->addWidget(addHidden(m_bitrateBox));
+
+	    trackRowLayout->setWidth();
         }
 
         m_commentBox = new KEdit(this, "commentBox");
 	m_commentBox->setTextFormat(Qt::PlainText);
-	addItem(i18n("&Comment:"), m_commentBox, rightColumnLayout);
+	addItem(i18n("&Comment:"), m_commentBox, rightColumnLayout, "edit");
+    	fileNameLabel->setMinimumHeight(m_trackSpin->height());
+
     }
 
     connect(m_artistNameBox, SIGNAL(textChanged(const QString&)),
@@ -422,33 +587,68 @@ void TagEditor::save(const PlaylistItemList &list)
     if(!list.isEmpty() && m_dataChanged) {
 	
 	KApplication::setOverrideCursor(Qt::waitCursor);
+	m_dataChanged = false;
+	m_performingSave = true;
 
 	// To keep track of the files that don't cooperate...
 
 	QStringList errorFiles;
+
+	// The list variable can become corrupted if the playlist holding its
+	// items dies, which is possible as we edit tags.  So we need to copy
+	// the end marker.
+
+	PlaylistItemList::ConstIterator end = list.end();
 	
-	for(PlaylistItemList::ConstIterator it = list.begin(); it != list.end(); ++it) {
+	for(PlaylistItemList::ConstIterator it = list.begin(); it != end; /* Deliberatly missing */ ) {
+
+	    // Process items before we being modifying tags, as the dynamic
+	    // playlists will try to modify the file we edit if the tag changes
+	    // due to our alterations here.
+
+	    kapp->eventLoop()->processEvents(QEventLoop::ExcludeUserInput);
+
 	    PlaylistItem *item = *it;
 	    
-	    QFileInfo newFile(item->dirPath() + QDir::separator() + m_fileNameBox->text());
-	    QFileInfo directory(item->dirPath());
-	    
+	    // The playlist can be deleted from under us if this is the last
+	    // item and we edit it so that it doesn't match the search, which
+	    // means we can't increment the iterator, so let's do it now.
+
+	    ++it;
+
+	    QFileInfo newFile(item->file().fileInfo().dirPath() + QDir::separator() +
+			      m_fileNameBox->text());
+	    QFileInfo directory(item->file().fileInfo().dirPath());
+
 	    // If (the new file is writable or the new file doesn't exist and
 	    // it's directory is writable) and the old file is writable...  
 	    // If not we'll append it to errorFiles to tell the user which
 	    // files we couldn't write to.
 	    
 	    if(item &&
-	       item->tag() &&
-	       (newFile.isWritable() || (!newFile.exists() && directory.isWritable())) &&
-	       item->isWritable())
+	       item->file().tag() &&
+	       item->file().fileInfo().isWritable())
 	    {
 		
 		// If the file name in the box doesn't match the current file
 		// name...
 		
-		if(list.count() == 1 && item->fileName() != newFile.fileName()) {
+		if(list.count() == 1 &&
+		   item->file().fileInfo().fileName() != newFile.fileName())
+		{
 		    
+		    // We're definitely trying to rename a file, so if it exists
+		    // but isn't writable, or it doesn't exist but its directory
+		    // isn't writable, we have an error.
+
+		    if((newFile.exists() && !newFile.isWritable()) ||
+		       (!newFile.exists() && !directory.isWritable()) &&
+		       item)
+		    {
+			errorFiles.append(newFile.filePath());
+			continue;
+		    }
+
 		    // Rename the file if it doesn't exist or the user says
 		    // that it's ok.
 		    
@@ -459,8 +659,8 @@ void TagEditor::save(const PlaylistItemList &list)
 			   i18n("File Exists")) == KMessageBox::Yes)
 		    {
 			QDir currentDir;
-			currentDir.rename(item->filePath(), newFile.filePath());
-			item->setFile(newFile.filePath());
+			currentDir.rename(item->file().absFilePath(), newFile.filePath());
+			item->file().setFile(newFile.filePath());
 		    }
 		}
 		
@@ -471,37 +671,42 @@ void TagEditor::save(const PlaylistItemList &list)
 		// each field that we write.
 		
 		if(m_enableBoxes[m_artistNameBox]->isOn())
-		    item->tag()->setArtist(m_artistNameBox->currentText());
+		    item->file().tag()->setArtist(m_artistNameBox->currentText());
 		if(m_enableBoxes[m_trackNameBox]->isOn())
-		    item->tag()->setTitle(m_trackNameBox->text());
+		    item->file().tag()->setTitle(m_trackNameBox->text());
 		if(m_enableBoxes[m_albumNameBox]->isOn())
-		    item->tag()->setAlbum(m_albumNameBox->currentText());
-		if(m_enableBoxes[m_trackSpin]->isOn())
-		    item->tag()->setTrack(m_trackSpin->value());
-		if(m_enableBoxes[m_yearSpin]->isOn())
-		    item->tag()->setYear(m_yearSpin->value());
+		    item->file().tag()->setAlbum(m_albumNameBox->currentText());
+		if(m_enableBoxes[m_trackSpin]->isOn()) {
+		    if(m_trackSpin->text().isEmpty())
+			m_trackSpin->setValue(0);
+		    item->file().tag()->setTrack(m_trackSpin->value());
+		}
+		if(m_enableBoxes[m_yearSpin]->isOn()) {
+		    if(m_yearSpin->text().isEmpty())
+			m_yearSpin->setValue(0);
+		    item->file().tag()->setYear(m_yearSpin->value());
+		}
 		if(m_enableBoxes[m_commentBox]->isOn())
-		    item->tag()->setComment(m_commentBox->text());
+		    item->file().tag()->setComment(m_commentBox->text());
 		
 		if(m_enableBoxes[m_genreBox]->isOn())
-		    item->tag()->setGenre(m_genreBox->currentText());
+		    item->file().tag()->setGenre(m_genreBox->currentText());
 		
-		item->tag()->save();
+		item->file().tag()->save();
 		
-		item->slotRefresh();
+		item->refresh();
+		item->playlist()->update();
 	    }
 	    else if(item)
-		errorFiles.append(item->fileName());
-
-	    kapp->processEvents();
+		errorFiles.append(item->file().absFilePath());
 	}
 	
 	if(!errorFiles.isEmpty())
 	    KMessageBox::detailedSorry(this,
 				       i18n("Could not save to specified file(s)."), 
 				       i18n("Could Not Write to:\n") + errorFiles.join("\n"));
-	m_dataChanged = false;
-
+	CollectionList::instance()->dataChanged();
+	m_performingSave = false;
 	KApplication::restoreOverrideCursor();
     }
 }
@@ -513,8 +718,8 @@ void TagEditor::saveChangesPrompt()
 
     QStringList files;
 
-    for(PlaylistItemList::Iterator it = m_items.begin(); it != m_items.end(); it++)
-	files.append((*it)->fileName());
+    for(PlaylistItemList::Iterator it = m_items.begin(); it != m_items.end(); ++it)
+	files.append((*it)->file().absFilePath());
 
     if(KMessageBox::questionYesNoList(this,
 				      i18n("Do you want to save your changes to:\n"), 
@@ -528,26 +733,32 @@ void TagEditor::saveChangesPrompt()
     }
 }
 
-void TagEditor::addItem(const QString &text, QWidget *item, QBoxLayout *layout)
+void TagEditor::addItem(const QString &text, QWidget *item, QBoxLayout *layout, const QString &iconName)
 {
     if(!item || !layout)
 	return;
 
     QLabel *label = new QLabel(item, text, this);
-
+    QLabel *iconLabel = new QLabel(item, 0, this);
+    
+    if(!iconName.isNull())
+	iconLabel->setPixmap(SmallIcon(iconName));
+    
     QCheckBox *enableBox = new QCheckBox(i18n("Enable"), this);
     enableBox->setChecked(true);
 
     label->setMinimumHeight(enableBox->height());
 
     if(layout->direction() == QBoxLayout::LeftToRight) {
+    	layout->addWidget(iconLabel);
 	layout->addWidget(label);
 	layout->addWidget(item);
 	layout->addWidget(enableBox);
     }
     else {
 	QHBoxLayout *l = new QHBoxLayout(layout);
-
+	
+	l->addWidget(iconLabel);
 	l->addWidget(label);
 	l->setStretchFactor(label, 1);
 
@@ -567,13 +778,18 @@ void TagEditor::addItem(const QString &text, QWidget *item, QBoxLayout *layout)
 
 void TagEditor::showEvent(QShowEvent *e)
 {
-    slotRefresh();
+    if(m_collectionChanged) {
+	updateCollection();
+	slotRefresh();
+    }
+
     QWidget::showEvent(e);
 }
 
 bool TagEditor::eventFilter(QObject *watched, QEvent *e)
 {
-    if(watched->inherits("QSpinBox") && e->type() == QEvent::KeyRelease)
+    QKeyEvent *ke = static_cast<QKeyEvent*>(e);
+    if(watched->inherits("QSpinBox") && e->type() == QEvent::KeyRelease && ke->state() == 0)
 	slotDataChanged();
 
     return false;
@@ -595,4 +811,12 @@ void TagEditor::slotItemRemoved(PlaylistItem *item)
 	slotRefresh();
 }
 
+void TagEditor::slotPlaylistDestroyed(Playlist *p)
+{
+    if(m_currentPlaylist == p) {
+	m_currentPlaylist = 0;
+	slotSetItems(PlaylistItemList());
+    }
+}
+
 #include "tageditor.moc"
