@@ -14,27 +14,27 @@ class Mixer_ALSA : public Mixer
 		~Mixer_ALSA();
 		
 		virtual int  readVolumeFromHW( int devnum, Volume &vol );
-		virtual int  writeVolumeToHW( int devnum, Volume vol );
+		virtual int  writeVolumeToHW( int devnum, Volume &vol );
 		virtual bool setRecsrcHW( int devnum, bool on);
 		virtual bool isRecsrcHW( int devnum );
+	        virtual bool prepareUpdate();
+
 		
 	protected:
 		virtual int	openMixer();
 		virtual int releaseMixer();
 		
 	private:
-		snd_mixer_t *handle;
 		int identify( snd_mixer_selem_id_t *sid );
 		snd_mixer_elem_t* getMixerElem(int devnum);
-		QString mixer_card_name;
-		QString mixer_device_name;
+
 		virtual QString errorText(int mixer_error);
-		//typedef QValueList<snd_mixer_elem_t *> AlsaMixerElemList;
-		//AlsaMixerElemList mixer_elem_list;
 		typedef QValueList<snd_mixer_selem_id_t *>AlsaMixerSidList;
 		AlsaMixerSidList mixer_sid_list;
-		unsigned int *mixerIDs;
+
 		bool masterChosen;
+                bool _initialUpdate;
+		snd_mixer_t *_handle;
 };
 
 #endif
