@@ -1,7 +1,7 @@
 /*
  *              KMix -- KDE's full featured mini mixer
  *
- *              $Id: mixer_oss.cpp,v 1.36 2003/11/30 20:58:03 esken Exp $
+ *              $Id: mixer_oss.cpp,v 1.37.2.3 2004/11/23 21:07:47 helio Exp $
  *
  *              Copyright (C) 1996-2000 Christian Esken
  *                        esken@kde.org
@@ -79,18 +79,17 @@ Mixer* OSS_getMixer( int device, int card )
 {
   Mixer *l_mixer;
   l_mixer = new Mixer_OSS( device, card );
-  l_mixer->setupMixer();
   return l_mixer;
 }
 
+/*
 Mixer* OSS_getMixerSet( MixSet set, int device, int card )
 {
   Mixer *l_mixer;
   l_mixer = new Mixer_OSS( device, card );
-  l_mixer->setupMixer( set );
   return l_mixer;
 }
-
+*/
 
 Mixer_OSS::Mixer_OSS(int device, int card) : Mixer(device, card)
 {
@@ -284,20 +283,21 @@ int Mixer_OSS::readVolumeFromHW( int devnum, Volume &vol )
   else
     {
       vol.setVolume( Volume::LEFT, (volume & 0x7f));
-      if( vol.channels() > 1 )
+      if( vol.count() > 1 )
         vol.setVolume( Volume::RIGHT, ((volume>>8) & 0x7f));
-      return 0;
+      //fprintf(stderr, "Mixer_OSS::readVolumeFromHW(%i,vol) set vol %i %i\n", devnum,  vol.getVolume(Volume::LEFT), vol.getVolume(Volume::RIGHT));
+	return 0;
     }
 }
 
 
 
-int Mixer_OSS::writeVolumeToHW( int devnum, Volume vol )
+int Mixer_OSS::writeVolumeToHW( int devnum, Volume &vol )
 {
   int volume;
   if( vol.isMuted() ) volume = 0;
   else
-    if ( vol.channels() > 1 )
+    if ( vol.count() > 1 )
       volume = vol[ Volume::LEFT ] + ((vol[ Volume::RIGHT ])<<8);
     else
       volume = vol[ Volume::LEFT ];
