@@ -1,5 +1,5 @@
 /*
- * $Id: cdrom.c,v 1.26 2004/01/05 23:04:09 aseigo Exp $
+ * $Id: cdrom.c,v 1.34.2.1 2004/11/03 14:56:54 larkang Exp $
  *
  * This file is part of WorkMan, the civilized CD player library
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -27,8 +27,6 @@
  * platform- or drive-dependent code here.
  */
 
-static char cdrom_id[] = "$Id: cdrom.c,v 1.26 2004/01/05 23:04:09 aseigo Exp $";
-
 #include <errno.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -88,11 +86,11 @@ struct drivelist {
  */
 int intermittent_dev = 0;
 
-int wm_cd_cur_balance = 10;
-int wm_cur_cdmode = WM_CDM_UNKNOWN;
-char *wm_cd_device = NULL; /* do not use this extern */
+static int wm_cd_cur_balance = 10;
+static int wm_cur_cdmode = WM_CDM_UNKNOWN;
+static char *wm_cd_device = NULL; /* do not use this extern */
 
-struct wm_drive drive = {
+static struct wm_drive drive = {
   0,
   NULL,
   NULL,
@@ -167,6 +165,7 @@ int wm_cd_destroy( void )
   FREE(drive.vendor);
   FREE(drive.model);
   FREE(drive.revision);
+  drive.proto = NULL;
 
   return 0;
 }
@@ -242,7 +241,7 @@ read_toc( void )
   if(!drive.proto)
     return -1;
     
-  if(drive.proto->gen_get_trackcount &&
+  if(drive.proto && drive.proto->gen_get_trackcount &&
     (drive.proto->gen_get_trackcount)(&drive, &thiscd.ntracks) < 0) {
     return -1 ;
   }
@@ -272,7 +271,7 @@ read_toc( void )
   }
 
   for (i = 0; i < thiscd.ntracks; i++) {
-    if(drive.proto->gen_get_trackinfo &&
+    if(drive.proto && drive.proto->gen_get_trackinfo &&
       (drive.proto->gen_get_trackinfo)(&drive, i + 1, &thiscd.trk[i].data,
       &thiscd.trk[i].start) < 0) {
       return -1;
@@ -291,7 +290,7 @@ read_toc( void )
       thiscd.trk[i].track, thiscd.trk[i].start);
   }
 
-  if(drive.proto->gen_get_cdlen &&
+  if(drive.proto && drive.proto->gen_get_cdlen &&
      (drive.proto->gen_get_cdlen)(&drive, &thiscd.trk[i].start) < 0) {
     return -1;
   }
@@ -327,54 +326,18 @@ int
 wm_cd_status( void )
 {
   static int oldmode = WM_CDM_UNKNOWN;
-  int mode, err;
-
-  if(!drive.proto ||
-     wm_cur_cdmode == WM_CDM_EJECTED ||
-     wm_cur_cdmode == WM_CDM_DEVICECHANGED) {
-    /* device was be changed */
-
-    if(drive.proto && drive.proto->gen_close)
-      drive.proto->gen_close(&drive);
-
-    if (wm_cur_cdmode != WM_CDM_EJECTED)
-      wm_cur_cdmode = WM_CDM_UNKNOWN;
-  }
-
-  if(wm_cur_cdmode == WM_CDM_UNKNOWN || wm_cur_cdmode == WM_CDM_EJECTED) {
-    /*
-     * Open the drive.
-     * This returns >0 if the device isn't ready and <0 if error.
-     */
-    cur_pos_abs = cur_pos_rel = cur_frame = 0;
-    cur_pos_rel = cur_pos_abs = 0;
+  int mode, err, tmp;
 
+  if(!drive.proto) {
+    oldmode = WM_CDM_UNKNOWN;
     err = wmcd_open( &drive );
-
-    if (err < 0)
-      return WM_CDM_UNKNOWN;
-    if (err > 0)
-      return WM_CDM_NO_DISC;
-
-    if(read_toc())
-      return WM_CDM_NO_DISC;
-
-    /* fresh cdtext info */
-    get_glob_cdtext(&drive, 1);
-
-    wm_cur_cdmode = WM_CDM_STOPPED;
-    thiscd.curtrack = 0;
+    if (err < 0) {
+      wm_cur_cdmode = WM_CDM_UNKNOWN;
+      return err;
+    }
   }
 
-  /* If the user hit the stop button, don't pass PLAYING as oldmode.
-   * Likewise, if we've just started playing, don't remember that
-   * we were stopped before (or the state machine in get_drive_status
-   * can get confused.)
-   */
-  if( (wm_cur_cdmode == WM_CDM_STOPPED) || (wm_cur_cdmode == WM_CDM_PLAYING) )
-    oldmode = wm_cur_cdmode;
-
-  if(drive.proto->gen_get_drive_status &&
+  if(drive.proto && drive.proto->gen_get_drive_status &&
     (drive.proto->gen_get_drive_status)(&drive, oldmode, &mode, &cur_frame,
     &(thiscd.curtrack), &cur_index) < 0) {
     perror("WM gen_get_drive_status");
@@ -385,18 +348,23 @@ wm_cd_status( void )
       gen_status(mode), thiscd.curtrack, cur_frame);
   }
 
+  if(WM_CDS_NO_DISC(oldmode) && WM_CDS_DISC_READY(mode)) {
+    /* device changed */
+    thiscd.ntracks = 0;
+    if(read_toc() || 0 == thiscd.ntracks)
+    {
+      close(drive.fd);
+      drive.fd = -1;
+      mode = WM_CDM_NO_DISC;
+    }
+    else /* refresh cdtext info */
+      get_glob_cdtext(&drive, 1);
+
+    wm_lib_message(WM_MSG_LEVEL_DEBUG|WM_MSG_CLASS, "device status changed() from %s to %s\n",
+      gen_status(oldmode), gen_status(mode));
+  }
   oldmode = mode;
 
-  if((mode == WM_CDM_EJECTED || mode == WM_CDM_UNKNOWN) &&
-    (wm_cur_cdmode != WM_CDM_DEVICECHANGED)) {
-    wm_cur_cdmode = WM_CDM_EJECTED;
-    thiscd.curtrack = -1;
-    thiscd.length = thiscd.curtracklen = 1;
-    cur_pos_abs = cur_pos_rel = cur_frame = 0;
-  
-    return wm_cur_cdmode;
-  }
-  
   /*
    * it seems all driver have'nt state for stop
    */
@@ -410,18 +378,18 @@ wm_cd_status( void )
   case WM_CDM_PAUSED:
     cur_pos_abs = cur_frame / 75;
     /* search for right track */
-    for(thiscd.curtrack = thiscd.ntracks;
-        thiscd.curtrack > 1 && cur_frame < thiscd.trk[CARRAY(thiscd.curtrack)].start;
-        thiscd.curtrack--)
+    for(tmp = thiscd.ntracks;
+        tmp > 1 && cur_frame < thiscd.trk[CARRAY(tmp)].start;
+        tmp--)
       ;
-
+    thiscd.curtrack = tmp;
     /* Fall through */
 
 
   case WM_CDM_UNKNOWN:
     if (mode == WM_CDM_UNKNOWN)
     {
-      mode = WM_CDM_STOPPED;
+      mode = WM_CDM_NO_DISC;
       cur_lasttrack = cur_firsttrack = -1;
     }
     /* Fall through */
@@ -453,8 +421,10 @@ wm_cd_status( void )
     break;
   case WM_CDM_FORWARD:
   case WM_CDM_EJECTED:
+    wm_cur_cdmode = mode;
     break;	
   }
+  
   wm_lib_message(WM_MSG_LEVEL_DEBUG|WM_MSG_CLASS,
     "wm_cd_status returns %s\n", gen_status(wm_cur_cdmode));
   return wm_cur_cdmode;
@@ -465,7 +435,6 @@ wm_cd_getcurtrack( void )
 {
   if(WM_CDS_NO_DISC(wm_cur_cdmode))
     return 0;
-    
   return thiscd.curtrack;
 }
   
@@ -487,6 +456,15 @@ wm_cd_getcountoftracks( void )
   return thiscd.ntracks;
 }
 
+int wm_cd_gettracklen( int track )
+{
+  if (track < 1 ||
+      track > thiscd.ntracks ||
+      thiscd.trk == NULL)
+    return 0;
+
+  return thiscd.trk[CARRAY(track)].length;
+}
 
 /*
  * wm_cd_play(starttrack, pos, endtrack)
@@ -517,8 +495,8 @@ wm_cd_play( int start, int pos, int end )
   /*
    * handle as overrun
    */
-  if(start < real_start) start = real_end;
-  if(start > real_end) start = real_start;
+  if(start < real_start) start = real_start;
+  if(start > real_end) start = real_end;
   
   /*
    * Try to avoid mixed mode and CD-EXTRA data tracks
@@ -530,7 +508,7 @@ wm_cd_play( int start, int pos, int end )
   
   cur_firsttrack = start;
   cur_lasttrack = end;
-
+    
   wm_cd_play_chunk(thiscd.trk[CARRAY(start)].start + pos * 75, end = thiscd.ntracks ?
     thiscd.length * 75 : thiscd.trk[CARRAY(end)].start - 1, thiscd.trk[CARRAY(start)].start);
   /* So we don't update the display with the old frame number */
@@ -604,7 +582,7 @@ wm_cd_pause( void )
     return -1;
 
   if(WM_CDM_PLAYING == wm_cur_cdmode) {
-    if(drive.proto->gen_pause)
+    if(drive.proto && drive.proto->gen_pause)
       (drive.proto->gen_pause)(&drive);
 
     paused_pos = cur_pos_rel;
@@ -635,7 +613,7 @@ wm_cd_stop( void )
 
   if (status != WM_CDM_STOPPED) {
     
-    if(drive.proto->gen_stop)
+    if(drive.proto && drive.proto->gen_stop)
       (drive.proto->gen_stop)(&drive);
 
     status = wm_cd_status();
@@ -691,8 +669,9 @@ struct cdtext_info*
 wm_cd_get_cdtext( void )
 {
   int status;
-  
+
   status = wm_cd_status();
+  
   if(WM_CDS_NO_DISC(status))
     return NULL;
   
