@@ -1,7 +1,7 @@
 #ifndef WM_CDDB_H
 #define WM_CDDB_H
 /*
- * $Id: wm_cddb.h,v 1.2 2000/05/30 15:18:44 dfoerste Exp $
+ * $Id: wm_cddb.h,v 1.3.4.1 2004/11/08 06:56:35 mueller Exp $
  *
  * This file is part of WorkMan, the civilized CD player library
  * (c) 1991-1997 by Steven Grimm (original author)
@@ -27,16 +27,13 @@
 
 
 extern struct wm_cddb cddb;
-extern int cur_cddb_protocol;
-extern char *cur_cddb_server, *cur_cddb_mail_adress,
-	*cur_cddb_path_to_cgi, *cur_cddb_proxy_server;
 
-unsigned long cddb_discid();
-void cddb_struct2cur();
-void cddb_cur2struct();
-void cddb_select();
-void connect_cddb();
-void update_cddbserver();
-void cddb_request();
+unsigned long cddb_discid(void);
+void cddb_struct2cur(void);
+void cddb_cur2struct(void);
+void cddb_select(void);
+void connect_cddb(void);
+void update_cddbserver(void);
+void cddb_request(void);
 
 #endif /* WM_CDDB_H */
