@@ -1,8 +1,9 @@
 #ifndef OGGPLAYER_IMPL_H
-#define OGGPLAYER_IMPL_H "$Id: oggPlayObject_impl.h,v 1.1.1.1 2001/04/05 09:19:02 garbanzo Exp $"
+#define OGGPLAYER_IMPL_H "$Id: oggPlayObject_impl.h,v 1.4 2002/10/27 01:10:28 pfeiffer Exp $"
 
-#if defined(__GNU_LIBRARY__) && defined(_SEM_SEMUN_UNDEFINED)
+#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
 /* union semun is defined by including <sys/sem.h> */
+#else
 /* according to X/OPEN we have to define it ourselves */
 union semun {
 	int val;                    /* value for SETVAL */
@@ -22,12 +23,12 @@ class oggPlayObject_impl
  public:
 	oggPlayObject_impl();
 	~oggPlayObject_impl();
-	bool loadMedia(const string &filename);
-	string description();
+	bool loadMedia(const std::string &filename);
+	std::string description();
 	poTime currentTime();
 	poTime overallTime();
 	poCapabilities capabilities();
-	string mediaName();
+	std::string mediaName();
 	poState state();
 	void play();
 	void halt();
@@ -42,6 +43,7 @@ class oggPlayObject_impl
 
 	static const int BACKBUFSIZ=4096;
 	OggVorbis_File vf;
+	std::string currentFile;
 
 	inline float conv_16le_float(short x)
 		{ return static_cast<float>(x) / 32768.0; }
